#import <Foundation/Foundation.h>

/* We need some way to distinguish between plain ints and jsvals in type sigs,
   so we wrap them in this stupid union.
*/

/* Due to a compiler bug in Rhapsody DR2 we have to wrap the union in a struct.
   When it's fixed we'll be able to use jsvals as JSVals without casting.
*/
typedef struct _JSVal {
    union __JSVal {
        unsigned JS : 10;	/* these fields just mark this as a JSVal */
        unsigned  V : 6;
        unsigned  a : 7;
        unsigned  l : 1;
        jsval v;		/* this is the only useful field */
    } u;
} JSVal;

static inline JSVal
_JSVal (jsval v) {
    JSVal V;

    V.u.v = v;
    return V;
}
#define JSVal(v) _JSVal(v)
#define jsval(V) (V).u.v

@interface ITKJSVal : NSObject
{
@public;
    jsval v;	/* v will not be GC'ed while the ITKJSVal exists */
}

+ (JSVal) autorelease: (JSVal) val;
+ (JSObject *) autoreleaseJSObject: (JSObject *) obj;
+ (JSString *) autoreleaseJSString: (JSString *) str;
+ (ITKJSVal *) jsVal: (JSVal) val;
+ (ITKJSVal *) jsObject: (JSObject *) obj;
+ (ITKJSVal *) jsString: (JSString *) str;
+ (ITKJSVal *) jsValWithBool: (BOOL) b;
+ (ITKJSVal *) jsValWithInt: (int) n;
+ (ITKJSVal *) jsValWithDouble: (double) d;
- (ITKJSVal *) initWithJSVal: (JSVal) val;
- (ITKJSVal *) initWithJSObject: (JSObject *) obj;
- (ITKJSVal *) initWithJSString: (JSString *) str;
- (ITKJSVal *) initWithBool: (BOOL) b;
- (ITKJSVal *) initWithInt: (int) n;
- (ITKJSVal *) initWithDouble: (double) d;
- (JSVal) jsVal;
- (JSObject *) jsObject;
- (JSString *) jsString;
- (BOOL) boolValue;
- (int) intValue;
- (double) doubleValue;
@end

@interface NSObject (JSObjCAdditions)
+ objectFromJSVal: (JSVal) v;
- (JSVal) idJSVal;
- (JSVal) gcIdJSVal; /* like idJSVal, but retains self until GC */
- (JSVal) jsVal;     /* returns [self idJSVal], but overridden by subclasses */
@end

@interface NSString (JSObjCAdditions)
+ (NSString *) stringWithJSVal: (JSVal) v;
+ (NSString *) stringWithJSObject: (JSObject *) obj;
+ (NSString *) stringWithJSString: (JSString *) str;
- (JSString *) jsString;
@end

