#--------------------------------------------------------------------------
#  itkBrowser.nib/Interpreter.tcl
#  Copyright (c) 1997-98, AAA+ Software (info@aaa-plus.com)
#  All rights reserved.
#  Author: R. Staringer
#--------------------------------------------------------------------------

# get an object's class without sending it a message
proc class {obj} {
    objc:peek [objc:iVarPtr $obj isa] Class
}

# get superclass of a class without sending it a message
proc superclass {class} {
    objc:peek [objc:iVarPtr $class super_class] Class
}

# strip the type information from a list returned by objc:info
proc namesonly {list} {
    set result {}
    set i 0
    foreach item $list {
        if {[objc:isType $item]} {
            lappend result anonymous#$i
        } else {
            lappend result [lindex $item 1]
        }
        incr i
    }
    return $result
}

# determine allowed choices for the popup depending on the selection and type
proc choices {value type} {
    switch $type {
        class {list Subclasses {Instantiated Subclasses} {Instance Methods} \
                    {Class Methods} {Instance Variables} {Adopted Protocols} \
                    {Instances} {Class Objects}}
        protocol {list {Instance Methods} {Class Methods} {Adopted Protocols}}
        id {concat {Methods {Instance Variables} {Adopted Protocols}} \
            [if {![objc:isValue $value Class]} {concat \
                [if {[objc:info met -a $value keyEnumerator] != ""} {
                    concat Keys
                } elseif {[objc:info met -a $value objectEnumerator] != ""} {
                    concat Elements
                }] [if {[objc:info met -a $value description] != ""} {
                    concat Description
                }]}]}
        imethod - cmethod {list {Return Type} {Argument Types}}
        ivar {concat Type}
    }
}

# determine the contents, type, and title of the next column after a
# pointer or aggregate type column
proc ptrValue {ptrValue ptrType column} {
    global type title
    # the ptrType can be just a bit count
    if {![objc:isType $ptrType]} {
        set type($column) string
        set title($column) $ptrType
        list $ptrValue
    } else {switch -glob -- [objc:encode -skip $ptrType] \
        [_C_STRUCT_B]* - [_C_UNION_B]* - [_C_BFLD]* {
            set type($column) field
            namesonly [lassign $ptrType title($column)]
        } \
        \\[_C_ARY_B]* {
            set type($column) field
            set title($column) $ptrType
            lassign $ptrType dummy n
            set indices {}
            for {set i 0} {$i < $n} {incr i} {
                lappend indices $i
            }
            set indices
        } \
        [_C_CLASS]* - [_C_ID]* {
            set type($column) id
            set title($column) $ptrType
            set ptrValue
        } \
        [_C_PTR]* {
            set type($column) pointer
            set title($column) $ptrType
            list $ptrValue
        } \
        [_C_CHR]* {
            set type($column) string
            set title($column) $ptrType
            if {$ptrValue == [YES]} {
                concat {[YES]}
            } else {
                list $ptrValue
            }
        } \
        default {
            set type($column) string
            set title($column) $ptrType
            list $ptrValue
        }
    }
}

objc:teach $popup {

# add/remove popup choices
- void setTitles: {TclObj choices} {
    set i 0
    set count [$self numberOfItems]
    foreach choice $choices {
        if {$i < $count} {
            [$self itemAtIndex: $i] setTitle: [@ $choice]
        } else {
            $self addItemWithTitle: [@ $choice]
        }
        incr i
    }
    while {$count > $i} {
        $self removeItemAtIndex: $i
        incr count -1
    }
}

}

objc:teach $self {

# delegate method called for each browser cell
- void browser: sender willDisplayCell: cell
         atRow: {int row} column: {int column} {
    global contents type
    set value [lindex $contents($column) $row]
    $cell setStringValue: [@ $value]
    $cell setLeaf: [switch $type($column) {
        class - ivar - imethod - cmethod - protocol - field - key NO
        pointer - id {
            lassign [split $value @] ptrType ptrAddr
            if {$value == "nil" || $value == "" || $ptrAddr == 0 ||
                $ptrType == "void" || $ptrType == "struct"} YES NO
        }
        default YES
    }]
}

# delegate method called once for each browser column
- int browser: sender numberOfRowsInColumn: {int column} {
    global contents all popup pattern patcol lastchoices lastchoice type title
    if {$column == 0} {
        set parent nil
        set parentType class
        set choice 0
        set allflag 0
        set -a ""
        set title(0) {Root Classes}
    } else {
        incr column -1
        set parent [$sender selectionInColumn: $column]
        set parentType $type($column)
        incr column
        set choices [choices $parent $parentType]
        if {![info exists lastchoices] || $lastchoices != $choices} {
            $popup setTitles: [set lastchoices $choices]
            if {![info exists lastchoice($parentType)]} {
                set lastchoice($parentType) 0
            }
            if {$lastchoice($parentType) < [$popup numberOfItems]} {
                $popup selectItemAtIndex: $lastchoice($parentType)
            }
        }
        set choice [$popup indexOfSelectedItem]
        set lastchoice($parentType) $choice
        set title($column) [* [$popup titleOfSelectedItem]]
        if {[set allflag [$all intValue]]} {
            set -a "-a"
            switch [lindex $title($column) end] {
                Subclasses - Methods - Variables -
                Protocols - Instances - Objects {
                    set title($column) "All $title($column)"
                }
            }
        } else {
            set -a ""
        }
        if {[info exists patcol] && $patcol != $column} {
            $pattern setStringValue: [@ ""]
        }
    }
    set contents($column) [switch $parentType {
        class {
            switch -- $choice {
                0 {
                    set type($column) class
                    lsort [eval objc:info class ${-a} -sub $parent]
                }
                1 {
                    set type($column) class
                    set map($parent) ""
                    if {$allflag} {
                        foreach c [objc:info class -sub $parent -a] {
                            set map($c) $c
                        }
                    } else {
                        foreach c [objc:info class -sub $parent] {
                            set map($c) $c
                            foreach s [objc:info class -a -sub $c] {
                                set map($s) $c
                            }
                        }
                    }
                    set sub() ""
                    foreach id [objc:info id -a -i $parent] {
                        set sub($map([class $id])) ""
                    }
                    unset sub()
                    lsort [array names sub]
                }
                2 {
                    set type($column) imethod
                    lsort [namesonly [eval objc:info met ${-a} $parent]]
                }
                3 {
                    set type($column) cmethod
                    lsort [namesonly [eval objc:info met ${-a} -c $parent]]
                }
                4 {
                    set type($column) ivar
                    namesonly [eval objc:info iv ${-a} $parent]
                }
                5 {
                    set type($column) protocol
                    lsort [eval objc:info proto ${-a} $parent]
                }
                6 {
                    set type($column) id
                    set result [lsort [eval objc:info id ${-a} -i $parent]]
                    if {[set i [lsearch $result ITKClassBrowserCell@*]] == -1} {
                        set result
                    } else {
                        set n [llength [lmatch $result ITKClassBrowserCell@*]]
                        lreplace $result $i [expr {$i + $n - 1}]
                    }
                }
                7 {
                    set type($column) id
                    lrmdups [concat [eval objc:info id -c ${-a} -i $parent] \
                            [eval objc:info id -c ${-a} -i [class $parent]]]
                }
            }
        }
        protocol {
            switch -- $choice {
                0 {
                    set type($column) imethod
                    lsort [namesonly [eval objc:info met ${-a} -p $parent]]
                }
                1 {
                    set type($column) cmethod
                    lsort [namesonly [eval objc:info met ${-a} -c -p $parent]]
                }
                3 {
                    set type($column) protocol
                    lsort [eval objc:info proto ${-a} -p $parent]
                }
            }
        }
        ivar - imethod - cmethod {
            set flags -a
            switch $parentType {
                ivar {
                    set info iv
                }
                imethod {
                    set info met
                }
                cmethod {
                    set info met
                    lappend flags -c
                }
            }
            incr column -2
            set obj [$sender selectionInColumn: $column]
            if {$type($column) == "protocol"} {
                lappend flags -p
            }
            incr column 2
            set type($column) string
            lassign [eval objc:info $info $flags $obj $parent] types
            switch -- $choice {
                0 {
                    lrange $types 0 0
                }
                1 {
                    lrange $types 2 end
                }
            }
        }
        id {
            switch -- $choice {
                0 {
                    if {[objc:isValue $parent Class]} {
                            set type($column) cmethod
                        lsort [namesonly [eval objc:info met ${-a} -c $parent]]
                    } else {
                        set type($column) imethod
                        lsort [namesonly [eval objc:info met ${-a} $parent]]
                    }
                }
                1 {
                    set type($column) field
                    set fields [namesonly [lrange [objc:expandType \
                                            [objc:typeOfPtr $parent]] 1 end]]
                    if {$allflag || [objc:isValue $parent Class]} {
                        set fields
                    } else {
                        set offset [llength [objc:expandType \
                                                [superclass [class $parent]]]]
                        lrange $fields [expr {$offset - 1}] end
                    }
                }
                2 {
                    set type($column) protocol
                    lsort [eval objc:info proto ${-a} $parent]
                }
                3 {
                    if {[$parent respondsToSelector: keyEnumerator]} {
                        set type($column) key
                        set result {}
                        set enumerator [$parent keyEnumerator]
                        while {[set obj [$enumerator nextObject]] != "nil"} {
                            lappend result [* [$obj description]]
                        }
                        lsort $result
                    } elseif {[$parent respondsToSelector: objectEnumerator]} {
                        set type($column) id
                        set result {}
                        set enumerator [$parent objectEnumerator]
                        while {[set obj [$enumerator nextObject]] != "nil"} {
                            lappend result $obj
                        }
                        set result
                    } else {
                        set type($column) string
                        list [* [$parent description]]
                    }
                }
                4 {
                    set type($column) string
                    list [* [$parent description]]
                }
            }
        }
        pointer {
            ptrValue [peek $parent] [objc:typeOfPtr $parent] $column
        }
        field {
            for {set col [expr {$column - 1}]} \
                {$type($col) == "field"} {incr col -1} {
                lvarpush fields [$sender selectionInColumn: $col]
            }
            set ptr [$sender selectionInColumn: $col]
            set ptrValue [peek $ptr]
            set ptrType [objc:expandType [objc:typeOfPtr $ptr]]
            foreach field $fields {
                if {![scan $field anonymous#%d index]} {
                    set index [lsearch -exact \
                                [namesonly [lrange $ptrType 1 end]] $field]
                }
                switch -glob -- [objc:encode -skip $ptrType] \
                    [_C_STRUCT_B]* - [_C_BFLD]* {
                        set ptrValue [lindex $ptrValue $index]
                        incr index
                        set ptrType [lindex $ptrType $index]
                    } \
                    [_C_UNION_B]* {
                        incr index
                        set ptrValue [lindex $ptrValue $index]
                        set ptrType [lindex $ptrType $index]
                    } \
                    \\[_C_ARY_B]* {
                        set ptrValue [lindex $ptrValue $field]
                        set ptrType [lindex $ptrType 0]
                    }
                if {![objc:isType $ptrType]} {
                    lassign $ptrType ptrType
                }
            }
            ptrValue $ptrValue $ptrType $column
        }
        key {
            set type($column) id
            set title($column) $parent
            [$sender selectionInColumn: [expr {$column - 2}]] \
                objectForKey: [@ $parent]
        }
    }]
    if {[set pat [* [$pattern stringValue]]] != ""} {
        set contents($column) [lmatch -glob $contents($column) $pat]
    }
    set length [llength $contents($column)]
    if {$length >= 7} {
        append title($column) " \[$length\]"
    }
    return $length
}

# delegate method called after browser:numberOfRowsInColumn:
- browser: sender titleOfColumn: {int column} {
    global title
    @ $title($column)
}

}

objc:teach $browser {

- void action: sender {
    global displaysel text
    if {![$displaysel intValue]} {
        return
    }
    set range [$text rangeForUserTextChange]
    set string [@ [$self selectionInColumn: [$self selectedColumn]]]
    if {[$text shouldChangeTextInRange: $range replacementString: $string]} {
        [$text window] makeFirstResponder: $text
        $text insertText: $string
        $text didChangeText
        lassign $range location
        $text setSelectedRange: [list $location [$string length]]
    }
}

- void doubleAction: sender {
    global type text owner
    set s [$self selectionInColumn: [$self selectedColumn]]
    # FIXME -- need generic mechanism to translate id's to native syntax
    if {[* [$owner scriptingLanguage]] == "JavaScript"} {
	if {[objc:isValue $s Class]} {
	    if {[string first @ $s] != -1} {
		set s "(Class)[lindex [split $s @] end]"
	    }
	} elseif {[objc:isValue $s id]} {
	    lassign [split $s @] class addr
	    set s "($class *)$addr"
        }
    }
    set range [$text rangeForUserTextChange]
    set string [@ "$s "]
    if {[$text shouldChangeTextInRange: $range replacementString: $string]} {
        [$text window] makeFirstResponder: $text
        $text insertText: $string
        $text didChangeText
    }
}

- TclObj selectionInColumn: {int column} {
    global contents
    lindex $contents($column) [[$self matrixInColumn: $column] selectedRow]
}

}

objc:teach $enablerelease {

- void action: sender {
    NSAutoreleasePool enableRelease: [if {[$self intValue]} YES NO]
}

}

proc peek {ptr} {objc:peek -trust $ptr}

objc:teach $trust {

- void action: sender {
    if {[$self intValue]} {
        proc peek {ptr} {objc:peek -trust $ptr}
    } else {
        proc peek {ptr} {objc:peek $ptr}
    }
}

}

objc:teach $displaysel {

- void action: sender {
    global browser
    if {[$self intValue] && [$browser selectedColumn] != -1} {
        $browser action: $self
    }
}

}

objc:teach $autoupdate {

- void action: sender {
    upvar #0 self interp
    global update
    if {[$self intValue]} {
        set update [$interp eval: {$popup action: nil} every: 2]
    } elseif {[info exists update]} {
        $interp removeTimedEval: $update
        unset update
    }
}

}

objc:teach "$all $popup $pattern" {

- void action: sender {
    global browser title contents
    upvar #0 patcol column
    set column [$browser lastColumn]
    set oldcontents $contents($column)
    set clip [[$browser matrixInColumn: $column] superview]
    lassign [$clip bounds] origin
    $browser reloadColumn: $column
    # browser:titleOfColumn: is not called for existing columns
    if {[* [$browser titleOfColumn: $column]] != $title($column)} {
        $browser setTitle: [@ $title($column)] ofColumn: $column
    }
    if {$contents($column) == $oldcontents} {
        $clip scrollToPoint: $origin
        [$clip superview] reflectScrolledClipView: $clip
    }
}

}

# swap an empty view into the box containing the browser when its height
# becomes too small; otherwise the buttons above the browser will disappear
objc:teach [set browserView [[$browser superview] retain]] {

- void setFrameSize: {NSSize size} {
    lassign $size width height
    if {$height <= 45} {
        global emptyView
        [$self superview] setContentView: $emptyView
    } else {
        former setFrameSize: $size
    }
}

}

# swap the original view back into the box when the height gets large enough
objc:teach [set emptyView [[NSView alloc] initWithFrame: {{0 0} {0 0}}]] {

- void setFrameSize: {NSSize size} {
    lassign $size width height
    if {$height > 45} {
        global browserView
        [$self superview] setContentView: $browserView
    } else {
        former setFrameSize: $size
    }
}

}

set text [$owner commandTextView]

# insert the box containing the browser into the command window's splitview
set splitView [[[$text superview] superview] superview]
set old [[$splitView subviews] objectAtIndex: 0]
set new [$browserView superview]
$new setFrameSize: [lindex [$old frame] 1]
$splitView replaceSubview: $old with: $new
$window release

# to avoid an infinitely growing instances list, we use a special cell class
# whose instances are not shown by the browser.
if {[objc:info class ITKClassBrowserCell] == ""} {
    objc:newClass ITKClassBrowserCell NSBrowserCell
}
$browser setCellClass: ITKClassBrowserCell
$browser setDelegate: $self
$browser setTakesTitleFromPreviousColumn: [NO]
$browser setMaxVisibleColumns: 4
$browser setMinColumnWidth: 170
$browser loadColumnZero

proc cleanup {obj} {
    global browser browserView emptyView update self
    # stop receiving delegate messages
    $browser setDelegate: nil
    # release saved content views
    $browserView release
    $emptyView release
    # stop auto-update
    if {[info exists update]} {
        $self removeTimedEval: $update
    }
    # release the interpreter
    $self autorelease
}
objc:deallocTrace $owner cleanup
