/******************************************************************************
*
* (C) 1994 AAA+ Software Forschungs- & Entwicklungs-GmbH.
* All rights reserved.
* 
*******************************************************************************
*
* DESCRIPTION:
*
* External itkObjC interface.
*
******************************************************************************/

#ifndef _ITKOBJC_H
#define _ITKOBJC_H

#import <itkTcl/tcl.h>
#import <Foundation/Foundation.h>
#import <itkObjCRuntime/itkObjCRuntimeTypes.h>

#ifndef _ITKOBJC_INLINE
#define _ITKOBJC_INLINE
#endif

typedef Tcl_Obj *TclObj;

extern TclObj ItkObjCNullObj;
extern NSString *ItkObjCTclErrorException;

#define ITKOBJC_NSEXCEPTION 43

#define ItkObjC_Init Itkobjc_Init
#define ItkObjC_SafeInit Itkobjc_SafeInit

/* Prototypes for all external functions */

int ItkObjC_Init (Tcl_Interp *);
int ItkObjC_SafeInit (Tcl_Interp *);

void ItkObjC_SetIdCheckingEnabled (int);
int ItkObjC_IsIdCheckingEnabled (void);
_ITKOBJC_INLINE int ItkObjC_IsIdRegistered (id);
int ItkObjC_StringToId (char *, id *);
char *ItkObjC_IdToString (id, Tcl_DString *, int);

char *ItkObjC_StringToType (const char *, Tcl_DString *);
char *ItkObjC_TypeToString (const char *, Tcl_DString *, int);

const TclObj ItkObjC_NullObj (void);
#define ItkObjC_IsNullObj(tclObj) (!(tclObj) || (tclObj) == ItkObjCNullObj)
TclObj ItkObjC_NewIdObj (id);
void ItkObjC_SetIdObj (TclObj, id);
int ItkObjC_GetIdFromObj (Tcl_Interp *, TclObj, id *);
TclObj ItkObjC_NewSelObj (SEL);
void ItkObjC_SetSelObj (TclObj, SEL);
int ItkObjC_GetSelFromObj (Tcl_Interp *, TclObj, SEL *);
TclObj ItkObjC_NewPtrObj (TclObj, void *);
void ItkObjC_SetPtrObj (TclObj, TclObj, void *);
int ItkObjC_GetPtrFromObj (Tcl_Interp *, TclObj, TclObj *, void **);
TclObj ItkObjC_NewTypeObj (const char *);
void ItkObjC_SetTypeObj (TclObj, const char *);
int ItkObjC_GetTypeFromObj (Tcl_Interp *, TclObj, const char **);
TclObj ItkObjC_NewLongLongObj (long long);
void ItkObjC_SetLongLongObj (TclObj, long long);
int ItkObjC_GetLongLongFromObj (Tcl_Interp *, TclObj, long long *);

TclObj ItkObjC_GetObjFromBinary (void *, const char *, ItkObjCPromote);
int ItkObjC_GetBinaryFromObj (Tcl_Interp *, TclObj, const char *,
			      ItkObjCPromote, void *);

TclObj ItkObjC_GetListObjFromNSArray (NSArray *, id);
int ItkObjC_GetNSArrayFromListObj (Tcl_Interp *, TclObj, id, NSArray **);

int ItkObjC_ObjEqual (TclObj, TclObj);
int ItkObjC_ObjCmp (TclObj, TclObj);

int ItkObjC_SetObjResultFromNSException (Tcl_Interp *, NSException *);
NSException *ItkObjC_GetNSExceptionFromObjResult (Tcl_Interp *, int);
const NSString *ItkObjC_TclErrorException (void);

int ItkObjC_AddClass (Tcl_Interp *, TclObj, Class, TclObj, Class *);
void ItkObjC_RemoveClass (Class);

int ItkObjC_Teach (Tcl_Interp *, TclObj, char, SEL,
		   const char *, TclObj, TclObj);
int ItkObjC_Unteach (Tcl_Interp *, TclObj, char, SEL);

int ItkObjC_MsgSend (ItkObjCMsgKind, Tcl_Interp *, int, TclObj *);
int ItkObjC_Call (ItkObjCFunction *, Tcl_Interp *, int, TclObj *);
void ItkObjC_SetTclTclTypeCheckingEnabled (int);
int ItkObjC_IsTclTclTypeCheckingEnabled (void);

int ItkObjC_LinkVar (Tcl_Interp *, TclObj, void *, const char *);
Tcl_Command ItkObjC_LinkFunction (Tcl_Interp *, TclObj, void *, const char *);

int ItkObjC_LoadPrecomp (const char *);
void ItkObjC_DefineMacro (const char *, const char *, int);
void ItkObjC_UndefMacro (const char *);
const char *ItkObjC_ExpandMacro (const char *, const char **, int *);
void ItkObjC_Typedef (const char *, const char *);
const char *ItkObjC_ExpandTypedef (const char *);

Tcl_Command ItkObjC_ResolveFunction (Tcl_Interp *, TclObj);

id ItkObjC_LookUpProtocol (const char *);
int ItkObjC_AddProtocol (Tcl_Interp *, TclObj, TclObj, TclObj, id *);

@interface NSObject (ItkObjCAdditions)
+ objectFromTclIdObj: (TclObj) tclObj;
- (TclObj) tclIdObj;
@end

@interface NSArray (ItkObjCAdditions)
+ arrayWithTclListObj: (TclObj) tclObj;
+ arrayWithTclListObj: (TclObj) tclObj replaceNullObjWith: nullObj;
- (TclObj) tclListObj;
- (TclObj) tclListObjReplaceWithNullObj: nullObj;
@end

@interface NSString (ItkObjCAdditions)
+ stringWithTclObj: (TclObj) tclObj;
- (TclObj) tclObj;
@end

@interface NSValue (ItkObjCAdditions)
+ valueWithTclObj: (TclObj) tclObj;
- (TclObj) tclObjValue;
@end

@interface ITKTclObj : NSObject
{
    TclObj tclObj;
}

+ (TclObj) autorelease: (TclObj) aTclObj;
+ tclObj: (TclObj) aTclObj;
- initWithTclObj: (TclObj) aTclObj;
- (TclObj) tclObj;
@end

#endif
