#--------------------------------------------------------------------------
#  SimpleBrowser.nib/Interpreter.tcl
#  Copyright (c) 1997, AAA+ Software (info@aaa-plus.com)
#  All rights reserved.
#  Authors: W. Pitzeier, R. Staringer
#
#  You may freely copy, distribute and reuse the code in this example.
#  AAA+ disclaims any warranty of any kind, expressed or implied,
#  as to its fitness for any particular use.
#--------------------------------------------------------------------------

# This is a simplified version of the class browser contained in every Joy
# command window. The source code for the real thing is in
# /Local/Library/Frameworks/itkRuntime.framework/Resources/itkBrowser.nib/Interpreter.tcl.

# teach the two skills needed for a passive browser delegate
objc:teach $self {

# skill that gets called each time a new cell is displayed
- void   browser: sender willDisplayCell: cell
           atRow: {int row} column: {int column} {
    global contents
    set class [lindex $contents($column) $row]
    $cell setStringValue: [@ $class]
    # if the class has no subclasses it is a leaf of the browser
    $cell setLeaf: [if {[objc:info class -sub $class] == ""} YES NO]
}

# skill that gets called each time a new column is displayed
- int browser: sender numberOfRowsInColumn: {int column}
{
    global contents
    if {$column == 0} {
        # leftmost column holds rootclasses
        set superclass nil
    } else {
        # get the selected value in the left neighboring column
        incr column -1
        set superclass [lindex $contents($column) \
                               [[$sender matrixInColumn: $column] selectedRow]]
        incr column
    }
    llength [set contents($column) [lsort [objc:info class -sub $superclass]]]
}

}

$browser setDelegate: $self
# for window resizing:
# if the window width gets to small,
# reduze the number of displayed columns
$browser setMinColumnWidth: 120
# if the window width gets larger, display more columns
$browser setMaxVisibleColumns: 5
