#--------------------------------------------------------------------------
#  JoyTerminal.nib/Interpreter.tcl
#  Copyright (c) 1998, AAA+ Software (info@aaa-plus.com)
#  All rights reserved.
#  Author: W. Pitzeier
#
#  You may freely copy, distribute and reuse the code in this example.
#  AAA+ disclaims any warranty of any kind, expressed or implied,
#  as to its fitness for any particular use.
#--------------------------------------------------------------------------

objc:teach $NSApp {

    # open new joy command window
    - void newCommandWindow: sender {
        if {[* [[JoyInfoController sharedController] commandLanguage]] == "Tcl"} {
            set interp [[ITKTclInterp alloc] init]
        } else {
            set interp [[ITKJSInterp alloc] init]
        }
        set window [$interp commandWindow]

        # normally a Joy command window does not have a menu bar on
        # the Windows platform, but this application consists of command 
        # windows only, so we need it.  Has no effect on Next and Rhapsody.
        $window setMenu: [$self mainMenu]
        
        $window setExcludedFromWindowsMenu: [NO]

        set keyWindow [$self keyWindow]
        if {$keyWindow == "nil"} {
            # center first window
            $window center
        } else {
            # cascade additional windows
            set frame [$keyWindow frame]
            lassign $frame origin size
            lassign $origin originX originY
            lassign $size width height
            set maxY [expr $originY + $height]
            set point "$originX $maxY"
            set point [$window cascadeTopLeftFromPoint: $point]
            $window setFrameTopLeftPoint: $point
        }

        $interp showCommandWindow: $sender
    }

    - void showInfoPanel: sender {
        if {[catch {$::NSApp orderFrontStandardAboutPanel: $sender}] && ![info exists ::infoPanel]} {
            NSBundle loadNibNamed: [@ InfoPanel] owner: $self
            $::infoPanel center
        }
        $::infoPanel makeKeyAndOrderFront: $sender
    }
    
    - void showPreferences: sender {
        [JoyInfoController sharedController] showPreferencesPanel: $sender
    }
    
    # opens this source file
    - void openSource: sender {
        set path [[[NSBundle mainBundle] pathForResource: [@ JoyTerminal] ofType: [@ nib]] stringByAppendingPathComponent: [@ Interpreter.tcl]]
        set workspace [NSWorkspace sharedWorkspace]
        if {![$workspace openFile: $path]} {
            # If no application opens a file with that extension we open
            # it with TextEdit as default application.  This only
            # happens on Windows.
            $workspace openFile: $path withApplication: [@ $::env(NEXT_ROOT)/NextDemos/TextEdit.app/TextEdit]
        }
    }
    
    # opens all nib files of this application (for all languages)
    - void openNibs: sender {
        foreach language [portableGlob [* [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent: [@ *.lproj]]]] {
            foreach nibFile [portableGlob [* [[@ $language] stringByAppendingPathComponent: [@ *.nib]]]] {
                # we need to open the file inside the nib since Windows
                # can not open a folder with an application.
                [NSWorkspace sharedWorkspace] openFile: [[@ $nibFile] stringByAppendingPathComponent: [@ objects.nib]]
            }
        }
    }

    - void closeModalPanel: sender {
         $self stopModalWithCode: [$sender tag]
    }

}


objc:teach ITKInterp {
    # source Tcl or JavaScript commands into the key command window
    - void sourceCommands: sender {
        set types [NSArray arrayWithObject: [$self scriptFileExtension]]
        set openPanel [NSOpenPanel openPanel]
        $openPanel setAllowsMultipleSelection: [YES]
        if {[$openPanel runModalForTypes: $types] == [NSOKButton]} {
            objc:foreach filename [$openPanel filenames] {
                $self source: [$filename stringByDeletingPathExtension]
            }
        }
    }

    # print the key window's content
    - void printWindow: sender {
        [$self commandTextView] print: $sender
    }

    # change the title of the key window
    - void setWindowTitle: sender {
        set window [$self commandWindow]
        lassign [joy::promptPanel "Set Window Title" "New Title:" [* [$window title]] "Set"] button title
        if {$button} {
            $window setTitle: [@ $title]
        }
    }
}

objc:teach "ITKTclInterp ITKJSInterp" {
    # free memory of closed command window's interpreter
    - BOOL windowShouldClose: window {
        set result [former windowShouldClose: $window]
        if {$result == $self || $result} {
            $self autorelease
            return [YES]
        }
        return [NO]
    }
}

# portable version of glob needed for Windows,
# since glob only allows slash path separators
proc portableGlob pattern {
    glob [join [split $pattern \\] /]
}

# application initialization
ITKJSInterp mainInterp
$NSApp newCommandWindow: $self
