###############################################################################
#
# (C) 1998 AAA+ Software Forschungs- & Entwicklungs-GmbH.
# All rights reserved.
#
###############################################################################
#
# DESCRIPTION:
#
# Tcl code sourced into IB by the main interp of the itkDeveloper.palette
# and joyDeveloper.palette.
#
###############################################################################

#######################################################################
#                          Inspector Stuff
#######################################################################

objc:teach ITKInterp {

- editorClassName { @ ITKInterpEditor }

}

#######################################################################
#                         Images
#######################################################################

objc:teach ITKInterp {

- id imageForViewer {
    set bundle [NSBundle bundleForClass: itkDeveloper]
    set file [$bundle pathForImageResource: [@ [$self class]]]
    if {$file != "nil"} {
        return [[NSImage alloc] initByReferencingFile: $file]
    }
    return nil
}

}

#######################################################################
#                         Interpreter init files
#######################################################################

objc:declare STR {tmpnam tcl_tmpnam} {STR}

objc:teach $self {

- void willSaveDocument: {{NSNotification *} note} {
    set document [$note object]
    upvar #0 ${document}(tmpPath) tmpPath
    set scriptFiles  [glob -nocomplain -- \
                       [file join [* [$document documentPath]] *.{tcl,js}]]
    if {$scriptFiles  != ""} {
        set tmpPath [tmpnam [NULL]]
        file mkdir $tmpPath
        eval [list file copy --] $scriptFiles  [list $tmpPath]
    }
}

- void didSaveDocument: {{NSNotification *} note} {
    set document [$note object]
    upvar #0 ${document}(tmpPath) tmpPath
    if {[info exists tmpPath]} {
        set scriptFiles  [glob -nocomplain [file join $tmpPath *]]
        eval [list file copy --] $scriptFiles  [list [* [$document documentPath]]]
        file delete -force $tmpPath
        unset tmpPath
    }
}

- void showHelp: sender {
	[NSWorkspace sharedWorkspace] openFile: [[NSBundle bundleWithPath: [$self documentationPath]] pathForResource: [@ Default] ofType: [@ html]]
}

}

$defaultCenter addObserver: $self \
                  selector: willSaveDocument: \
                      name: [@ IBWillSaveDocumentNotification] \
                    object: nil
$defaultCenter addObserver: $self \
                  selector: didSaveDocument: \
                      name: [@ IBDidSaveDocumentNotification] \
                    object: nil

objc:teach ITKTclInterp {

- void unparseInitFile: file {
    global NSApp
    set f [open [* $file] w]
    puts $f "# Joy interpreter init file created automatically, \
             [clock format [clock seconds] -format %c]\n"
    set conns [[$NSApp activeDocument] itkConnectorsForSource: $self]
    set objVars self
    set objs $self
    set otherVars ""
    objc:foreach conn $conns {
        set var [* [$conn sourceAspect]]
        if {[* [$conn destinationAspect]] == "self"} {
            if {![lcontain $objVars $var]} {
                lappend objVars $var
                lappend objs [$conn destination]
            }
        } else {
            if {![lcontain $otherVars $var]} {
                lappend otherVars $var
            }
        }
    }
    set globals [union $objVars $otherVars]
    lvarpop globals [lsearch -exact $globals self]
    foreach v $objVars o $objs {
        puts $f "objc:teach \$$v {\n"
        catch {$o unparseTclSkillsTo: $f globals: $globals}
        puts $f "}\n"
    }
    close $f
}

}

objc:teach ITKJSInterp {

- void unparseInitFile: file {
    global NSApp
    set f [open [* $file] w]
    puts $f "/* Joy interpreter init file created automatically, \
             [clock format [clock seconds] -format %c] */\n"
    set conns [[$NSApp activeDocument] itkConnectorsForSource: $self]
    objc:foreach conn $conns {
        if {[* [$conn destinationAspect]] == "self"} {
            catch {[$conn destination] \
                    unparseJSSkillsTo: $f var: [* [$conn sourceAspect]]}
        }
    }
    close $f
}

}

objc:teach NSObject {

- void unparseTclSkill: {TclObj desc} to: {TclObj f} globals: {TclObj globals} {
    puts $f "$desc {"
    puts $f "    global $globals"
    puts $f "\n}\n"
}

- void unparseTclSkillsTo: {TclObj f} globals: {TclObj globals} {}

- void unparseJSSkillsTo: {TclObj f} var: {TclObj var} {}

}

objc:teach {NSControl NSActionCell} {

- void unparseTclSkillsTo: {TclObj f} globals: {TclObj globals} {
    super unparseTclSkillsTo: $f globals: $globals
    $self unparseTclSkill: "- void action: sender" to: $f globals: $globals
    if {[$self respondsToSelector: doubleAction]} {
        $self unparseTclSkill: "- void doubleAction: sender" \
                           to: $f \
                      globals: $globals
    }
}

- void unparseJSSkillsTo: {TclObj f} var: {TclObj var} {
    puts $f "$var\[\"action:\"\] = function() {\n    \n}\n"
    if {[$self respondsToSelector: doubleAction]} {
        puts $f "$var\[\"doubleAction:\"\] = function() {\n    \n}\n"
    }
}

}

########################################################################
#                             Show/Hide Command Window
########################################################################

objc:teach ITKInterp {

- void willBeginTestingInterface: {{NSNotification *} note} {
    iset visible [[$self commandWindow] isVisible]
}

- void didEndTestingInterface: {{NSNotification *} note} {
    if {[iset visible]} {
        [$self commandWindow] makeKeyAndOrderFront: $self
    } else {
        [$self commandWindow] close
    }
}

}

$defaultCenter addObserver: $self \
                  selector: willBeginTestingInterface: \
                      name: [@ IBWillBeginTestingInterfaceNotification] \
                    object: nil

$defaultCenter addObserver: $self \
                  selector: didEndTestingInterface: \
                      name: [@ IBDidEndTestingInterfaceNotification] \
                    object: nil

$defaultCenter addObserver: [ITKJSInterp mainInterp] \
                  selector: willBeginTestingInterface: \
                      name: [@ IBWillBeginTestingInterfaceNotification] \
                    object: nil

$defaultCenter addObserver: [ITKJSInterp mainInterp] \
                  selector: didEndTestingInterface: \
                      name: [@ IBDidEndTestingInterfaceNotification] \
                    object: nil

#######################################################################
#                             Menu entries
#######################################################################

objc:teach $NSApp {

- void newJoyApplication: sender {
    global tcl_platform
    # Determine the new application's interface style
    set interfaceStyle [[[[IBDocument alloc] init] autorelease] interfaceStyle]
    if {$interfaceStyle == "[NSNextStepInterfaceStyle]"} {
        set template next
    } elseif {$interfaceStyle == "[NSWindows95InterfaceStyle]"} {
        set template windows
    } else {
        set template mac
    }
    if {[* [[JoyInfoController sharedController] commandLanguage]] == "JavaScript"} {
        append template -JS
    }
    if {[IBDocument instancesRespondToSelector: initWithContentsOfFile:]} {
        set init initWithContentsOfFile:
    } else {
	set init initFromPath:
    }
    # Load template nib
    set document [[IBDocument alloc] $init [[NSBundle bundleForClass: itkDeveloper] \
                    pathForResource: [@ NewJoyApplication-$template] ofType: [@ nib]]]

    # Make it a brand new untitled and unedited document
    $document setUntitled: [YES]
    $document setDocumentPath: [@ UNTITLED]
    $document edit
    if {[$document respondsToSelector: setDocumentEdited:]} {
        $document setDocumentEdited: [NO]
    }
}

}


# Create Joy menu ...
set _joyMenu [[NSMenu alloc] initWithTitle: [@ Joy]]
# ... and make it submenu of the mainMenu
set _item [[$NSApp mainMenu] \
             insertItemWithTitle: [@ Joy] \
                          action: 0 \
                   keyEquivalent: [@ ""] \
                         atIndex: 5]
[$NSApp mainMenu] setSubmenu: $_joyMenu forItem: $_item
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

# Add the entries in Joy menu.

# Create Info menu ...
set _infoMenu [[NSMenu alloc] initWithTitle: [@ Info]]
# ... and make it submenu of the Joy menu
set _item [$_joyMenu \
             addItemWithTitle: [@ Info] \
                       action: 0 \
                keyEquivalent: [@ ""]]
$_joyMenu setSubmenu: $_infoMenu forItem: $_item
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

if {$tcl_platform(os) == "OPENSTEP"} {
    set _infoPanelTitle "Info Panel..."
} else {
    set _infoPanelTitle "About Joy..."
}        
set _item [$_infoMenu \
                addItemWithTitle: [@ $_infoPanelTitle] \
                          action: showInfoPanel: \
                   keyEquivalent: [@ ""]]
$_item setTarget: [JoyInfoController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0
unset _infoPanelTitle

set _item [$_infoMenu \
                addItemWithTitle: [@ "Preferences..."] \
                          action: showPreferencesPanel: \
                   keyEquivalent: [@ ""]]
$_item setTarget: [JoyInfoController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_infoMenu \
                addItemWithTitle: [@ "Registration..."] \
                          action: showLicensingInfoPanel: \
                   keyEquivalent: [@ ""]]
$_item setTarget: [JoyLicensingController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_infoMenu \
                addItemWithTitle: [@ "Help..."] \
                          action: showHelp: \
                   keyEquivalent: [@ ""]]
$_item setTarget: $self
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
                addItemWithTitle: [@ "Command Window..."] \
                          action: showCommandWindow: \
                   keyEquivalent: [@ J]]
$_item setTarget: [JoyInfoController sharedController]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 8

set _item [$_joyMenu \
                addItemWithTitle: [@ "Save Commands..."] \
                          action: saveCommands: \
                   keyEquivalent: [@ ""]]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
                addItemWithTitle: [@ "Clear Command Window"] \
                          action: clearCommandWindow: \
                   keyEquivalent: [@ k]]
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
                addItemWithTitle: [@ "New Joy Application"] \
                          action: newJoyApplication: \
                   keyEquivalent: [@ ""]]
$_item setTarget: $NSApp
$_item setEnabled: [YES]
$_item setMnemonicLocation: 0

set _item [$_joyMenu \
                addItemWithTitle: [@ "Save Nib as App..."] \
                          action: 0 \
                   keyEquivalent: [@ A]]
$_item setMnemonicLocation: 12

unset _item
unset _joyMenu
unset _infoMenu

#######################################################################
#                             User Startup
########################################################################

if {[file readable ~/.itkDeveloperInit.tcl]} {
    if {[catch {source ~/.itkDeveloperInit.tcl} msg]} {
        puts ".itkDeveloperInit.tcl: $msg"
    }
}
